using Adaptation.Eaf.Management.ConfigurationData.CellAutomation;
using Adaptation.Ifx.Eaf.EquipmentConnector.File.Configuration;
using Adaptation.Shared;
using Adaptation.Shared.Duplicator;
using Adaptation.Shared.Methods;
using Adaptation.Shared.Metrology;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.Json;

namespace Adaptation.FileHandlers.IQSSi;

public class FileRead : Shared.FileRead, IFileRead
{

    private readonly string _IQSFile;

    public FileRead(ISMTP smtp, Dictionary<string, string> fileParameter, string cellInstanceName, int? connectionCount, string cellInstanceConnectionName, FileConnectorConfiguration fileConnectorConfiguration, string equipmentTypeName, string parameterizedModelObjectDefinitionType, IList<ModelObjectParameterDefinition> modelObjectParameters, string equipmentDictionaryName, Dictionary<string, List<long>> dummyRuns, Dictionary<long, List<WS.Results>> staticRuns, bool useCyclicalForDescription, bool isEAFHosted) :
        base(new Description(), false, smtp, fileParameter, cellInstanceName, connectionCount, cellInstanceConnectionName, fileConnectorConfiguration, equipmentTypeName, parameterizedModelObjectDefinitionType, modelObjectParameters, equipmentDictionaryName, dummyRuns, staticRuns, useCyclicalForDescription, isEAFHosted: connectionCount is null)
    {
        _MinFileLength = 10;
        _NullData = string.Empty;
        _Logistics = new(this);
        if (_FileParameter is null)
            throw new Exception(cellInstanceConnectionName);
        if (_ModelObjectParameterDefinitions is null)
            throw new Exception(cellInstanceConnectionName);
        if (!_IsDuplicator)
            throw new Exception(cellInstanceConnectionName);
        _IQSFile = GetPropertyValue(cellInstanceConnectionName, modelObjectParameters, "IQS.File");
    }

    void IFileRead.Move(Tuple<string, Test[], JsonElement[], List<FileInfo>> extractResults, Exception exception)
    {
        bool isErrorFile = exception is not null;
        if (!isErrorFile && !string.IsNullOrEmpty(_Logistics.ReportFullPath))
        {
            FileInfo fileInfo = new(_Logistics.ReportFullPath);
            if (fileInfo.Exists && fileInfo.LastWriteTime < fileInfo.CreationTime)
                File.SetLastWriteTime(_Logistics.ReportFullPath, fileInfo.CreationTime);
        }
        Move(extractResults);
    }

    void IFileRead.WaitForThread() => WaitForThread(thread: null, threadExceptions: null);

    string IFileRead.GetEventDescription()
    {
        string result = _Description.GetEventDescription();
        return result;
    }

    List<string> IFileRead.GetHeaderNames()
    {
        List<string> results = _Description.GetHeaderNames();
        return results;
    }

    string[] IFileRead.Move(Tuple<string, Test[], JsonElement[], List<FileInfo>> extractResults, string to, string from, string resolvedFileLocation, Exception exception)
    {
        string[] results = Move(extractResults, to, from, resolvedFileLocation, exception);
        return results;
    }

    JsonProperty[] IFileRead.GetDefault()
    {
        JsonProperty[] results = _Description.GetDefault(this, _Logistics);
        return results;
    }

    Dictionary<string, string> IFileRead.GetDisplayNamesJsonElement()
    {
        Dictionary<string, string> results = _Description.GetDisplayNamesJsonElement(this);
        return results;
    }

    List<IDescription> IFileRead.GetDescriptions(IFileRead fileRead, List<Test> tests, IProcessData processData)
    {
        List<IDescription> results = _Description.GetDescriptions(fileRead, _Logistics, tests, processData);
        return results;
    }

    Tuple<string, Test[], JsonElement[], List<FileInfo>> IFileRead.GetExtractResult(string reportFullPath, string eventName)
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        if (string.IsNullOrEmpty(eventName))
            throw new Exception();
        _ReportFullPath = reportFullPath;
        DateTime dateTime = DateTime.Now;
        results = GetExtractResult(reportFullPath, dateTime);
        if (results.Item3 is null)
            results = new Tuple<string, Test[], JsonElement[], List<FileInfo>>(results.Item1, Array.Empty<Test>(), JsonSerializer.Deserialize<JsonElement[]>("[]"), results.Item4);
        if (results.Item3.Length > 0 && _IsEAFHosted)
            WritePDSF(this, results.Item3);
        UpdateLastTicksDuration(DateTime.Now.Ticks - dateTime.Ticks);
        return results;
    }

    Tuple<string, Test[], JsonElement[], List<FileInfo>> IFileRead.ReExtract()
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        List<string> headerNames = _Description.GetHeaderNames();
        Dictionary<string, string> keyValuePairs = _Description.GetDisplayNamesJsonElement(this);
        results = ReExtract(this, headerNames, keyValuePairs);
        return results;
    }

    private static string GetLines(Logistics logistics, List<pcl.Description> descriptions)
    {
        StringBuilder result = new();
        pcl.Description x = descriptions[0];
        char del = '\t';
        _ = result.Append(x.AreaCountAvg).Append(del).                   // 001 - AreaCountAvg
            Append(x.AreaCountMax).Append(del).                      // 002 - AreaCountMax
            Append(x.AreaCountMin).Append(del).                      // 003 - AreaCountMin
            Append(x.AreaCountStdDev).Append(del).                   // 004 - AreaCountStdDev
            Append(x.AreaTotalAvg).Append(del).                      // 005 - AreaTotalAvg
            Append(x.AreaTotalMax).Append(del).                      // 006 - AreaTotalMax
            Append(x.AreaTotalMin).Append(del).                      // 007 - AreaTotalMin
            Append(x.AreaTotalStdDev).Append(del).                   // 008 - AreaTotalStdDev
            Append(x.Date).Append(del).                              // 009 - 
            Append(x.HazeAverageAvg).Append(del).                    // 010 - Haze Average
            Append(x.HazeAverageMax).Append(del).                    // 011 - 
            Append(x.HazeAverageMin).Append(del).                    // 012 - 
            Append(x.HazeAverageStdDev).Append(del).                 // 013 - 
            Append(x.HazeRegionAvg).Append(del).                     // 014 - 
            Append(x.HazeRegionMax).Append(del).                     // 015 - 
            Append(x.HazeRegionMin).Append(del).                     // 016 - 
            Append(x.HazeRegionStdDev).Append(del).                  // 017 - 
            Append(x.Lot).Append(del).                               // 018 - 
            Append(x.LPDCM2Avg).Append(del).                         // 019 - 
            Append(x.LPDCM2Max).Append(del).                         // 020 - 
            Append(x.LPDCM2Min).Append(del).                         // 021 - 
            Append(x.LPDCM2StdDev).Append(del).                      // 022 - 
            Append(x.LPDCountAvg).Append(del).                       // 023 - 
            Append(x.LPDCountMax).Append(del).                       // 024 - 
            Append(x.LPDCM2Min).Append(del).                         // 025 - 
            Append(x.LPDCountStdDev).Append(del).                    // 026 - 
            Append(x.Employee).Append(del).                          // 027 - 
            Append(x.RDS).Append(del).                               // 028 - Lot
            Append(x.Reactor).Append(del).                           // 029 - Process
            Append(x.Recipe.Replace(";", string.Empty)).Append(del). // 030 - Part
            Append(x.ScratchCountAvg).Append(del).                   // 031 - Scratch Count
            Append(x.ScratchCountMax).Append(del).                   // 032 - 
            Append(x.ScratchCountMin).Append(del).                   // 033 - 
            Append(x.ScratchTotalStdDev).Append(del).                // 034 - 
            Append(x.ScratchTotalAvg).Append(del).                   // 035 - Scratch Length
            Append(x.ScratchTotalMax).Append(del).                   // 036 - 
            Append(x.ScratchTotalMin).Append(del).                   // 037 - 
            Append(x.ScratchTotalStdDev).Append(del).                // 038 - 
            Append(x.SumOfDefectsAvg).Append(del).                   // 039 - Average Sum of Defects
            Append(x.SumOfDefectsMax).Append(del).                   // 040 - Max Sum of Defects
            Append(x.SumOfDefectsMin).Append(del).                   // 041 - Min Sum of Defects
            Append(x.SumOfDefectsStdDev).Append(del).                // 042 - SumOfDefectsStdDev
            Append(logistics.MesEntity).Append(del).           // 043 - 
            AppendLine();
        return result.ToString();
    }

    private void SaveIQSFile(string reportFullPath, DateTime dateTime, List<pcl.Description> descriptions, Test[] tests)
    {
        bool isDummyRun = false;
        List<(Shared.Properties.IScopeInfo, string)> collection = new();
        string lines = GetLines(_Logistics, descriptions);
        string successDirectory = _FileConnectorConfiguration.AlternateTargetFolder;
        string duplicateDirectory = Path.Combine(_FileConnectorConfiguration.SourceFileLocation, _CellInstanceName);
        if (!Directory.Exists(duplicateDirectory))
            _ = Directory.CreateDirectory(duplicateDirectory);
        string duplicateFile = Path.Combine(duplicateDirectory, Path.GetFileName(reportFullPath));
        if (tests.Length != 0)
        {
            ScopeInfo scopeInfo = new(tests[0], _IQSFile);
            if (!string.IsNullOrEmpty(lines))
                collection.Add(new(scopeInfo, lines));
        }
        WaitForFileConsumption(dateTime, descriptions, isDummyRun, successDirectory, duplicateDirectory, collection, duplicateFile);
    }

    private void FileCopy<T>(string reportFullPath, DateTime dateTime, List<T> descriptions) where T : Shared.Properties.IDescription
    {
        bool isDummyRun = false;
        string successDirectory = string.Empty;
        List<(Shared.Properties.IScopeInfo, string)> collection = new();
        string duplicateDirectory = Path.Combine(_FileConnectorConfiguration.SourceFileLocation, _CellInstanceName);
        if (!Directory.Exists(duplicateDirectory))
            _ = Directory.CreateDirectory(duplicateDirectory);
        string duplicateFile = Path.Combine(duplicateDirectory, Path.GetFileName(reportFullPath));
        string lines = File.ReadAllText(reportFullPath);
        lines = lines
            .Replace("ppm\t", "\t")
            .Replace("mm2\t", "\t")
            .Replace("um\t", "\t")
            .Replace("mm\t", "\t")
            .Replace("nm\t", "\t")
            .Replace("\t[ ", "\t")
            .Replace("\t*0", "\t")
            .Replace("\t*", "\t")
            .Replace("%\t", "\t")
            .Replace("]\t", "\t");
        File.WriteAllText(duplicateFile, lines);
        WaitForFileConsumption(dateTime, descriptions, isDummyRun, successDirectory, duplicateDirectory, collection, duplicateFile);
    }

    private Tuple<string, Test[], JsonElement[], List<FileInfo>> GetExtractResult(string reportFullPath, DateTime dateTime)
    {
        Tuple<string, Test[], JsonElement[], List<FileInfo>> results;
        ProcessDataStandardFormat processDataStandardFormat = ProcessDataStandardFormat.GetProcessDataStandardFormat(reportFullPath);
        _Logistics = new Logistics(reportFullPath, processDataStandardFormat);
        SetFileParameterLotIDToLogisticsMID();
        JsonElement[] jsonElements = ProcessDataStandardFormat.GetArray(processDataStandardFormat);
        List<pcl.Description> descriptions = pcl.ProcessData.GetDescriptions(jsonElements);
        Test[] tests = (from l in descriptions select (Test)l.Test).ToArray();
        results = new Tuple<string, Test[], JsonElement[], List<FileInfo>>(string.Join(Environment.NewLine, processDataStandardFormat.Logistics[0]), tests, jsonElements, new List<FileInfo>());
        if (_IsEAFHosted && _FileConnectorConfiguration.FileScanningIntervalInSeconds > 0)
            SaveIQSFile(reportFullPath, dateTime, descriptions, tests);
        if (_IsEAFHosted && _FileConnectorConfiguration.FileScanningIntervalInSeconds > 0)
            FileCopy(reportFullPath, dateTime, descriptions);
        return results;
    }

}